// Only for LINUX (DEBIAN) / FOX BOARD G20 - ACME SYSTEMS
// ALESSIO LOMBARDO 5B INFORMATICA 2012/2013 - ITIS VITTORIO EMANUELE III - PALERMO (ITALY)

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <signal.h>
#include <time.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <termios.h>
#include <fcntl.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <mysql/mysql.h>
#include "/usr/include/linux/i2c-dev.h"

#define DCMOTOR1   111   //J7.12
#define DCMOTOR2   109   //J7.14
#define REVCOUNTER 110   //J7.11
#define SMOTORA1   65    //J7.10
#define SMOTORA2   67    //J7.8
#define SMOTORB1   83    //J7.4
#define SMOTORB2   81    //J7.6
#define SONARA1    77    //J6.15
#define SONARA2    95    //J6.19
#define SONARB1    63    //J6.21
#define SONARB2    85    //J6.17
#define SOUND      76    //J6.16
#define ENABLE12V  108   //J7.13
#define RLEDR      57    //J7.38
#define RLEDG      60    //J7.35
#define RLEDB      58    //J7.37
#define LLEDR      88    //J7.34
#define LLEDG      86    //J7.32
#define LLEDB      59    //J7.36
#define BIGLED     43    //J6.38
#define PROXNE     89    //J7.33
#define PROXNW     87    //J7.31
#define PROXSE     64    //J7.9
#define PROXSW     82    //J7.3
#define BATTST     66    //J7.7
#define BMP085_ADDRESS       0x77  // I2C address of BMP085
#define LIS331DLH_ADDRESS    0x18  // I2C address of LIS331DLH
#define L3G4200D_ADDRESS     0x68  // I2C address of L3G4200D
#define HMC5883L_ADDRESS     0x1E  // I2C address of HMC5883L
#define SRF08_ADDRESS        0x70  // I2C address of SRF08
#define TIME       10000
#define TIMEDC     900000

struct structIMU{
   float X;
   float Y;
   float Z;
   float gyro_X; //deg/sec
   float gyro_Y;
   float gyro_Z;
   float acc_X;  //g
   float acc_Y;
   float acc_Z;
   float mag_X;
   float mag_Y;
   float mag_Z;
};
typedef struct structIMU IMUtype;

struct structGPS{
   char nmea[75];
   int quality;
   char time_hms[9];
   float latitude;
   char latitude_d;
   float latitude_g;
   float longitude;
   char longitude_d;
   float longitude_g;
   float altitude;
   int satellite;
};
typedef struct structGPS GPStype;

//TAP sensors global variable
const unsigned char OSS = 0;  // Oversampling Setting
static int ac1;
static int ac2; 
static int ac3; 
static unsigned int ac4;
static unsigned int ac5;
static unsigned int ac6;
static int b1; 
static int b2;
static int mb;
static int mc;
static int md;
static long b5; 
static int temp_u;
static unsigned long press_u;

/*OPEN A GPIO PORT*/
void gpio_open(int gpio,char direction){
   FILE *fpopen,*fpdir;
   char dir[50];  
   if((fpopen=fopen("/sys/class/gpio/export","w"))==NULL){
      printf("Error to export. GPIO ID: %d\n",gpio);
      //exit(1);
   }else{       
      fprintf(fpopen,"%d",gpio);
      fclose(fpopen);      
      sprintf(dir,"/sys/class/gpio/gpio%d/direction",gpio);
      if((fpdir=fopen(dir,"w"))==NULL){
         printf("Error to set the direction. GPIO ID: %d.\n",gpio);
         //exit(1);
      }else{
         if(direction=='i'){
            fprintf(fpdir,"in");
         }else if(direction=='o'){
            fprintf(fpdir,"out");
         }else{
            printf("Error: invalid direction. 'i' = in  'o' = out.\n");
         }
         fclose(fpdir);
      } 
   }
}
/*WRITE VALUE TO GPIO PORT*/
int gpio_wvalue(int gpio,int value){
   FILE *fpval;
   char val[50];   
   sprintf(val,"/sys/class/gpio/gpio%d/value",gpio);
   if(((fpval=fopen(val,"w"))==NULL)||((value!=0)&&(value!=1))){
      printf("Error to set the value. GPIO ID: %d\n",gpio);
      return -1;
   }else{
      fprintf(fpval,"%d",value);
      fclose(fpval);
   }
   return 0;
}
/*READ VALUE FROM GPIO PORT*/
int gpio_rvalue(int gpio){
   FILE *fpval;
   int value=0;
   char val[50];   
   sprintf(val,"/sys/class/gpio/gpio%d/value",gpio);
   if((fpval=fopen(val,"r"))==NULL){
      printf("Error to read the value. GPIO ID: %d\n",gpio);
      return -1;
   }else{
      fscanf(fpval,"%d",&value);
      fclose(fpval);
   }
   return value;
}
/*OPEN A I2C PORT*/
int openi2c(void){
   int fd;
   char filename[20];
   sprintf(filename, "/dev/i2c-0");
   fd = open(filename, O_RDWR);
   if(fd<0){
      printf("Error on open i2c \n");
      //exit(1);
   }
   return fd;	
}
/*CLOSE A I2C PORT*/
int closei2c(int fd){
   return close(fd);
}
/*SET A I2C PORT*/
void setdevicei2c(int fd, char address){
   if(ioctl(fd, I2C_SLAVE, address) < 0){
      printf("Error i2c on slave address\n");
      //exit(1);
   }
}
/*WRITE VALUE TO I2C PORT*/
void writebytei2c(int fd, char reg, char data){
   char buf[2];
   buf[0] = reg;
   buf[1] = data;
   if((write(fd,buf,2))!=2){
      printf("Error i2c send the read command\n");
      //exit(1);
   }
}
/*READ VALUE FROM I2C PORT*/
char readbytei2c(int fd, char reg){
   char buf[1];
   buf[0] = reg;
   if((write(fd,buf,1))!=1){
      printf("Error i2c on select the Byte\n");
      //exit(1);
   }
   if ((read(fd,buf,1))!=1) {
      printf("Error i2c on read the Byte\n");
      //exit(1);
   }
   return buf[0];
}
/*CLOSE ROBOT MANAGER*/
void closerobot(int fd_i2c,int *pos,int webcam_time,int webcam_fps,int webcam_resolution,char webcam_res[]){
   closei2c(fd_i2c);
   FILE *fd_motor;
   FILE *fd_webcam;
   if((fd_motor=fopen("save/motor","w"))!=NULL){
      fprintf(fd_motor,"%d,%d",pos[0],pos[1]);
      fclose(fd_motor);
   }else{
      printf("Errore: Impossibile salvare le posizioni dei motori\n");
   }
   if((fd_webcam=fopen("save/webcam","w"))!=NULL){
      fprintf(fd_webcam,"%d,%d,%d,%s",webcam_time,webcam_fps,webcam_resolution,webcam_res);
      fclose(fd_webcam);
   }else{
      printf("Errore: Impossibile salvare la configurazione della webcam\n");
   }
}
/*START VIDEO STREAMING FROM WEBCAM*/
void webcam_video(char webcam_res[],int webcam_fps){
   char string[200];
   sprintf(string,"/root/mjpg-streamer/mjpg_streamer -i '/root/mjpg-streamer/input_uvc.so -f %d -r %s' -o '/root/mjpg-streamer/output_http.so -w /var/www/webcam -p 4300' 2>/dev/null &",webcam_fps,webcam_res);  
   system(string);
}
/*CAPTURE IMAGE FROM WEBCAM*/
void webcam_picture(int time, char webcam_res[]){
   char string[200];
   sprintf(string,"/root/mjpg-streamer/mjpg_streamer -i '/root/mjpg-streamer/input_uvc.so -f 10 -r %s' -o '/root/mjpg-streamer/output_file.so -f /var/www/webcam/pics -d %d' 2>/dev/null &",webcam_res,time);  
   system(string);
}
/*START VIDEO STREAMING AND CAPTURE IMAGE FROM WEBCAM*/
void webcam_video_picture(int time, char webcam_res[]){
   char string[300];
   sprintf(string,"/root/mjpg-streamer/mjpg_streamer -i '/root/mjpg-streamer/input_uvc.so -f 10 -r %s' -o '/root/mjpg-streamer/output_http.so -w /var/www/webcam -p 4300' -o '/root/mjpg-streamer/output_file.so -f /var/www/webcam/pics -d %d' 2>/dev/null &",webcam_res,time);  
   system(string);
}
/*SHUTDOWN WEBCAM DEAMON*/
int webcam_off(){
   int ret=system("kill -9 $(pidof mjpg_streamer) 2> /dev/null");
   return WEXITSTATUS(ret);
}
/*DELETE IMAGE*/
int webcam_deletep(){
   int ret=system("rm /var/www/pics/*.* 2> /dev/null");
   return WEXITSTATUS(ret);
}
/*START UP MOTORS*/
void e12volt_on(){
   gpio_open(ENABLE12V,'o');
   gpio_wvalue(ENABLE12V,0);
   gpio_open(DCMOTOR1,'o');
   gpio_open(DCMOTOR2,'o');
   gpio_open(SMOTORA1,'o');
   gpio_open(SMOTORA2,'o');
   gpio_open(SMOTORB1,'o');
   gpio_open(SMOTORB2,'o');
   gpio_open(SONARA1,'o');
   gpio_open(SONARA2,'o');
   gpio_open(SONARB1,'o');
   gpio_open(SONARB2,'o');
   gpio_open(PROXNE,'i');
   gpio_open(PROXNW,'i');
   gpio_open(PROXSE,'i');
   gpio_open(PROXSW,'i');
   gpio_open(REVCOUNTER,'i');
   gpio_open(SOUND,'o');
   gpio_wvalue(DCMOTOR1,0);
   gpio_wvalue(DCMOTOR2,0);
   gpio_wvalue(SMOTORA1,0);
   gpio_wvalue(SMOTORA2,0);
   gpio_wvalue(SMOTORB1,0);
   gpio_wvalue(SMOTORB2,0);
   gpio_wvalue(SONARA1,0);
   gpio_wvalue(SONARA2,0);
   gpio_wvalue(SONARB1,0);
   gpio_wvalue(SONARB2,0);
}
/*TURN OFF MOTORS*/
void e12volt_off(){
   gpio_open(ENABLE12V,'o');
   gpio_wvalue(ENABLE12V,1);
}
/*CHANGE COLOR LED*/
int led(char selection,int color){
   int b[3];
   int i=0;
   int bigled=color;
   b[0]=0;
   b[1]=0;
   b[2]=0;
   if((color>0)&&(color<=7)){
      while(color>0){
         b[i]=color%2;
         color=color/2;
         i++;
      }
   }else if((color<0)||(color>7)){
      //printf("Error: Invalid color.\n");
      return -1;
   }
   if(selection=='r'){
      gpio_open(RLEDR,'o');
      gpio_open(RLEDG,'o');
      gpio_open(RLEDB,'o');
      gpio_wvalue(RLEDR,b[2]);
      gpio_wvalue(RLEDG,b[1]);
      gpio_wvalue(RLEDB,b[0]);
   }else if(selection=='l'){
      gpio_open(LLEDR,'o');
      gpio_open(LLEDG,'o');
      gpio_open(LLEDB,'o');
      gpio_wvalue(LLEDR,b[2]);
      gpio_wvalue(LLEDG,b[1]);
      gpio_wvalue(LLEDB,b[0]);
   }else if(selection=='b'){
      gpio_open(BIGLED,'o');
      if(gpio_wvalue(BIGLED,bigled)==-1){
         return -1;
      }
   }else{
      //printf("Error: Led not found.\n");
      return -2;
   }
   return 0;
}
/*CHANGE COLOR LED CASUALLY*/
void randomcolorled(int colorspeed){
   srand(time(NULL));
   while(1){
      led('l',(rand()%7)+1);
      led('r',(rand()%7)+1);
      usleep(colorspeed*1000);
   }
}
/*LED BLINK*/
void ledturnsignals(int turnsignal){
   led('r',0);
   led('l',0);
   while(1){
      if(turnsignal==1){
         led('r',6);
         usleep(300000);
         led('r',0);
         usleep(300000);
      }else if(turnsignal==2){
         led('l',6);
         usleep(300000);
         led('l',0);
         usleep(300000);
      }else if(turnsignal==3){
         led('r',6);
         led('l',6);
         usleep(300000);
         led('r',0);
         led('l',0);
         usleep(300000);
      }
   }
}
/*BATTERY STATUS*/
int battery_status(int *value){
   FILE *fpval;
   char val[50];   
   sprintf(val,"/sys/bus/platform/devices/at91_adc/chan1");
   if((fpval=fopen(val,"r"))==NULL){
      printf("Error to read the value.\n");
      //exit(1);
   }else{
      fscanf(fpval,"%d",value);
      fclose(fpval);
   }
   *value=((*value-600)*100)/(835-600);
   if(*value>100){
      *value=100;
   }
   gpio_open(BATTST,'i');
   return gpio_rvalue(BATTST);
}
/*SET DAISY7 MODULE*/
void daisy7_set(int fd){

   /*SET IMU SUB-MODULE*/
   setdevicei2c(fd, LIS331DLH_ADDRESS);
   writebytei2c(fd,0x20, 0x37);

   setdevicei2c(fd, L3G4200D_ADDRESS);
   writebytei2c(fd,0x20, 0xAF);
   writebytei2c(fd,0x21, 0x20);
   writebytei2c(fd,0x22, 0x00);
   writebytei2c(fd,0x23, 0x00);
   writebytei2c(fd,0x24, 0x00);

   setdevicei2c(fd, HMC5883L_ADDRESS);
   writebytei2c(fd,0x00,0x78);
   writebytei2c(fd,0x01,0x20);
   writebytei2c(fd,0x02,0x00);

   /*SET TAP SUB-MODULE*/
   unsigned char buf;
   setdevicei2c(fd, BMP085_ADDRESS);
   buf=readbytei2c(fd, 0xAA);
   ac1 = buf<<8;
   buf=readbytei2c(fd, 0xAB);
   ac1 |= buf;
   if(ac1>32768){
      ac1=ac1-65535;
   }
   buf=readbytei2c(fd, 0xAC);
   ac2 = buf<<8;
   buf=readbytei2c(fd, 0xAD);
   ac2 |= buf;
   if(ac2>32768){
      ac2=ac2-65535;
   }
   buf=readbytei2c(fd, 0xAE);
   ac3 = buf<<8;
   buf=readbytei2c(fd, 0xAF);
   ac3 |= buf;
   if(ac3>32768){
      ac3=ac3-65535;
   }
   buf=readbytei2c(fd, 0xB0);
   ac4 = buf<<8;
   buf=readbytei2c(fd, 0xB1);
   ac4 |= buf;
   buf=readbytei2c(fd, 0xB2);
   ac5 = buf<<8;
   buf=readbytei2c(fd, 0xB3);
   ac5 |= buf;
   buf=readbytei2c(fd, 0xB4);
   ac6 = buf<<8;
   buf=readbytei2c(fd, 0xB5);
   ac6 |= buf;
   buf=readbytei2c(fd, 0xB6);
   b1 = buf<<8;
   buf=readbytei2c(fd, 0xB7);
   b1 |= buf;
   if(b1>32768){
      b1=b1-65535;
   }
   buf=readbytei2c(fd, 0xB8);
   b2 = buf<<8;
   buf=readbytei2c(fd, 0xB9);
   b2 |= buf;
   if(b2>32768){
      b2=b2-65535;
   }
   buf=readbytei2c(fd, 0xBA);
   mb = buf<<8;
   buf=readbytei2c(fd, 0xBB);
   mb |= buf;
   if(mb>32767){
      mb=mb-65535;
   }
   buf=readbytei2c(fd, 0xBC);
   mc = buf<<8;
   buf=readbytei2c(fd, 0xBD);
   mc |= buf;
   if(mc>32768){
      mc=mc-65535;
   }
   buf=readbytei2c(fd, 0xBE);
   md = buf<<8;
   buf=readbytei2c(fd, 0xBF);
   md |= buf;
   if(md>32768){
      md=md-65535;
   }
}
/*READ TEMPERATURE, ALTITUDE, PRESSURE (DAISY7)*/
void readTAP(int fd, float *temperature, float *altitude, float *pressure){
   long x1, x2, x3, b3, b6, p;
   unsigned long b7;
   unsigned long b4;
   long x1p, x2p;
   unsigned char buf;
   setdevicei2c(fd, BMP085_ADDRESS);
   writebytei2c(fd,0xF4,0x2E);
   usleep(10000);
   setdevicei2c(fd, BMP085_ADDRESS);
   buf=readbytei2c(fd, 0xF6);
   temp_u = buf<<8;
   buf=readbytei2c(fd, 0xF7);
   temp_u |= buf;
   setdevicei2c(fd, BMP085_ADDRESS);
   writebytei2c(fd,0xF4,0x34 + (OSS<<6));
   usleep(10000);
   setdevicei2c(fd, BMP085_ADDRESS);
   buf=readbytei2c(fd, 0xF6);
   press_u = buf<<16;
   buf=readbytei2c(fd, 0xF7);
   press_u|= buf<<8;
   buf=readbytei2c(fd, 0xF8);
   press_u |= buf;
   press_u = press_u >> (8-OSS);
   x1p = (((long)temp_u - (long)ac6)*(long)ac5) >> 15;
   x2p = ((long)mc << 11)/(x1p + md);
   b5 = x1p + x2p;
   *temperature=((float)((b5 + 8)>>4))/((float)10);
   b6 = b5 - 4000;
   x1 = (b2 * (b6 * b6)>>12)>>11;
   x2 = (ac2 * b6)>>11;
   x3 = x1 + x2;
   b3 = (((((long)ac1)*4 + x3)<<OSS) + 2)>>2;
   x1 = (ac3 * b6)>>13;
   x2 = (b1 * ((b6 * b6)>>12))>>16;
   x3 = ((x1 + x2) + 2)>>2;
   b4 = (ac4 * (unsigned long)(x3 + 32768))>>15;
   b7 = ((unsigned long)(press_u - b3) * (50000>>OSS));
   if (b7 < 0x80000000){
      p = (b7<<1)/b4;
   }else{
      p = (b7/b4)<<1;
   }
   x1 = (p>>8) * (p>>8);
   x1 = (x1 * 3038)>>16;
   x2 = (-7357 * p)>>16;
   p += (x1 + x2 + 3791)>>4;
   *pressure =((float)p)/100;
   *altitude=(float)(44330*(1-pow((*pressure/1013.25),(1/5.255))));
}

/*FOR FILTER IMU FUNCTION*/
float invSqrt(float x) {
	float halfx = 0.5f * x;
	float y = x;
	long i = *(long*)&y;
	i = 0x5f3759df - (i>>1);
	y = *(float*)&i;
	y = y * (1.5f - (halfx * y * y));
	return y;
}

/*CONVERT IMU VALUES TO ROTATION VALUE (DAISY7)*/
void filterIMU(IMUtype *IMU, int reset){
   //float beta = 0.07f; // compute beta
   float beta = 0.1f;								// 2 * proportional gain (Kp)
   static float q0 = 1.0f, q1 = 0.0f, q2 = 0.0f, q3 = 0.0f;	// quaternion of sensor frame relative to auxiliary frame
   float sampleFreq = 512.0f;
   float recipNorm;
   float s0, s1, s2, s3;
   float qDot1, qDot2, qDot3, qDot4;
   float hx, hy;
   float _2q0mx, _2q0my, _2q0mz, _2q1mx, _2bx, _2bz, _4bx, _4bz, _2q0, _2q1, _2q2, _2q3, _2q0q2, _2q2q3, q0q0, q0q1, q0q2, q0q3, q1q1, q1q2, q1q3, q2q2, q2q3, q3q3;  
   if(reset==1){
      q0 = 1.0f;
      q1 = 0.0f;
      q2 = 0.0f;
      q3 = 0.0f;
   }
   // Rate of change of quaternion from gyroscope
   qDot1 = 0.5f * (-q1 * IMU->gyro_X - q2 *  IMU->gyro_Y - q3 *  IMU->gyro_Z);
   qDot2 = 0.5f * (q0 * IMU->gyro_X + q2 *  IMU->gyro_Z - q3 *  IMU->gyro_Y);
   qDot3 = 0.5f * (q0 * IMU->gyro_Y - q1 *  IMU->gyro_Z + q3 * IMU->gyro_X);
   qDot4 = 0.5f * (q0 * IMU->gyro_Z + q1 *  IMU->gyro_Y - q2 * IMU->gyro_X);
   // Compute feedback only if accelerometer measurement valid (avoids NaN in accelerometer normalisation)
   if(!((IMU->acc_X == 0.0f) && (IMU->acc_Y == 0.0f)&& ( IMU->acc_Z == 0.0f))) {
      // Normalise accelerometer measurement
      recipNorm = invSqrt( IMU->acc_X *  IMU->acc_X +  IMU->acc_Y *  IMU->acc_Y +  IMU->acc_Z *  IMU->acc_Z);
      IMU->acc_X *= recipNorm;
      IMU->acc_Y *= recipNorm;
      IMU->acc_Z *= recipNorm;   
      // Normalise magnetometer measurement
      recipNorm = invSqrt( IMU->mag_X *  IMU->mag_X + IMU->mag_Y * IMU->mag_Y + IMU->mag_Z * IMU->mag_Z);
      IMU->mag_X *= recipNorm;
      IMU->mag_Y *= recipNorm;
      IMU->mag_Z *= recipNorm;
      // Auxiliary variables to avoid repeated arithmetic
      _2q0mx = 2.0f * q0 *  IMU->mag_X;
      _2q0my = 2.0f * q0 * IMU->mag_Y;
      _2q0mz = 2.0f * q0 * IMU->mag_Z;
      _2q1mx = 2.0f * q1 *  IMU->mag_X;
      _2q0 = 2.0f * q0;
      _2q1 = 2.0f * q1;
      _2q2 = 2.0f * q2;
      _2q3 = 2.0f * q3;
      _2q0q2 = 2.0f * q0 * q2;
      _2q2q3 = 2.0f * q2 * q3;
      q0q0 = q0 * q0;
      q0q1 = q0 * q1;
      q0q2 = q0 * q2;
      q0q3 = q0 * q3;
      q1q1 = q1 * q1;
      q1q2 = q1 * q2;
      q1q3 = q1 * q3;
      q2q2 = q2 * q2;
      q2q3 = q2 * q3;
      q3q3 = q3 * q3;
      // Reference direction of Earth's magnetic field
      hx =  IMU->mag_X * q0q0 - _2q0my * q3 + _2q0mz * q2 +  IMU->mag_X * q1q1 + _2q1 * IMU->mag_Y * q2 + _2q1 * IMU->mag_Z * q3 -  IMU->mag_X * q2q2 -  IMU->mag_X * q3q3;
      hy = _2q0mx * q3 + IMU->mag_Y * q0q0 - _2q0mz * q1 + _2q1mx * q2 - IMU->mag_Y * q1q1 + IMU->mag_Y * q2q2 + _2q2 * IMU->mag_Z * q3 - IMU->mag_Y * q3q3;
      _2bx = sqrt(hx * hx + hy * hy);
      _2bz = -_2q0mx * q2 + _2q0my * q1 + IMU->mag_Z * q0q0 + _2q1mx * q3 - IMU->mag_Z * q1q1 + _2q2 * IMU->mag_Y * q3 - IMU->mag_Z * q2q2 + IMU->mag_Z * q3q3;
      _4bx = 2.0f * _2bx;
      _4bz = 2.0f * _2bz;
      // Gradient decent algorithm corrective step
      s0 = -_2q2 * (2.0f * q1q3 - _2q0q2 -  IMU->acc_X) + _2q1 * (2.0f * q0q1 + _2q2q3 -  IMU->acc_Y) - _2bz * q2 * (_2bx * (0.5f - q2q2 - q3q3) + _2bz * (q1q3 - q0q2) -  IMU->mag_X) + (-_2bx * q3 + _2bz * q1) * (_2bx * (q1q2 - q0q3) + _2bz * (q0q1 + q2q3) - IMU->mag_Y) + _2bx * q2 * (_2bx * (q0q2 + q1q3) + _2bz * (0.5f - q1q1 - q2q2) - IMU->mag_Z);
      s1 = _2q3 * (2.0f * q1q3 - _2q0q2 -  IMU->acc_X) + _2q0 * (2.0f * q0q1 + _2q2q3 -  IMU->acc_Y) - 4.0f * q1 * (1 - 2.0f * q1q1 - 2.0f * q2q2 -  IMU->acc_Z) + _2bz * q3 * (_2bx * (0.5f - q2q2 - q3q3) + _2bz * (q1q3 - q0q2) -  IMU->mag_X) + (_2bx * q2 + _2bz * q0) * (_2bx * (q1q2 - q0q3) + _2bz * (q0q1 + q2q3) - IMU->mag_Y) + (_2bx * q3 - _4bz * q1) * (_2bx * (q0q2 + q1q3) + _2bz * (0.5f - q1q1 - q2q2) - IMU->mag_Z);
      s2 = -_2q0 * (2.0f * q1q3 - _2q0q2 -  IMU->acc_X) + _2q3 * (2.0f * q0q1 + _2q2q3 -  IMU->acc_Y) - 4.0f * q2 * (1 - 2.0f * q1q1 - 2.0f * q2q2 -  IMU->acc_Z) + (-_4bx * q2 - _2bz * q0) * (_2bx * (0.5f - q2q2 - q3q3) + _2bz * (q1q3 - q0q2) -  IMU->mag_X) + (_2bx * q1 + _2bz * q3) * (_2bx * (q1q2 - q0q3) + _2bz * (q0q1 + q2q3) - IMU->mag_Y) + (_2bx * q0 - _4bz * q2) * (_2bx * (q0q2 + q1q3) + _2bz * (0.5f - q1q1 - q2q2) - IMU->mag_Z);
      s3 = _2q1 * (2.0f * q1q3 - _2q0q2 -  IMU->acc_X) + _2q2 * (2.0f * q0q1 + _2q2q3 -  IMU->acc_Y) + (-_4bx * q3 + _2bz * q1) * (_2bx * (0.5f - q2q2 - q3q3) + _2bz * (q1q3 - q0q2) -  IMU->mag_X) + (-_2bx * q0 + _2bz * q2) * (_2bx * (q1q2 - q0q3) + _2bz * (q0q1 + q2q3) - IMU->mag_Y) + _2bx * q1 * (_2bx * (q0q2 + q1q3) + _2bz * (0.5f - q1q1 - q2q2) - IMU->mag_Z);
      recipNorm = invSqrt(s0 * s0 + s1 * s1 + s2 * s2 + s3 * s3); // normalise step magnitude
      s0 *= recipNorm;
      s1 *= recipNorm;
      s2 *= recipNorm;
      s3 *= recipNorm;

      // Apply feedback step
      qDot1 -= beta * s0;
      qDot2 -= beta * s1;
      qDot3 -= beta * s2;
      qDot4 -= beta * s3;
   }
   // Integrate rate of change of quaternion to yield quaternion
   q0 += qDot1 * (1.0f / sampleFreq);
   q1 += qDot2 * (1.0f / sampleFreq);
   q2 += qDot3 * (1.0f / sampleFreq);
   q3 += qDot4 * (1.0f / sampleFreq);
   // Normalise quaternion
   recipNorm = invSqrt(q0 * q0 + q1 * q1 + q2 * q2 + q3 * q3);
   q0 *= recipNorm;
   q1 *= recipNorm;
   q2 *= recipNorm;
   q3 *= recipNorm;
   IMU->X = q1;
   IMU->Y = q2;
   IMU->Z = q3;
}
/*READ IMU VALUES (DAISY7)*/
struct structIMU readIMU(int fd, int reset){
   struct structIMU IMU;
   unsigned char buf;
   int temp,X,Y,Z;
   int sensivity_tmp=280;

   /*LIS331DLH*/
   setdevicei2c(fd, LIS331DLH_ADDRESS);
   buf=readbytei2c(fd, 0x29);
   X = buf<<8;
   buf=readbytei2c(fd, 0x28);
   X |= buf;
   if(X>32768){
      X=X-65535;
   }
   buf=readbytei2c(fd, 0x2B);
   Y = buf<<8;
   buf=readbytei2c(fd, 0x2A);
   Y |= buf;
   if(Y>32768){
      Y=Y-65535;
   }
   buf=readbytei2c(fd, 0x2D);
   Z = buf<<8;
   buf=readbytei2c(fd, 0x2C);
   Z |= buf;
   if(Z>32768){
      Z=Z-65535;
   }
   IMU.acc_X=(float)X/(float)1733;
   IMU.acc_Y=(float)Y/(float)1733;
   IMU.acc_Z=(float)Z/(float)1733;

   /*L3G4200D*/
   setdevicei2c(fd, L3G4200D_ADDRESS);
   temp = buf<<8;
   buf=readbytei2c(fd, 0x26);
   temp |= buf;
   buf=readbytei2c(fd, 0x29);
   X = buf<<8;
   buf=readbytei2c(fd, 0x28);
   X |= buf;
   if(X>32768){
      X=X-65535;
   }
   buf=readbytei2c(fd, 0x2B);
   Y = buf<<8;
   buf=readbytei2c(fd, 0x2A);
   Y |= buf;
   if(Y>32768){
      Y=Y-65535;
   }
   buf=readbytei2c(fd, 0x2D);
   Z = buf<<8;
   buf=readbytei2c(fd, 0x2C);
   Z |= buf;
   if(Z>32768){
      Z=Z-65535;
   }
   temp=temp/sensivity_tmp;
   IMU.gyro_X=(float)X*0.00875F;
   IMU.gyro_Y=(float)Y*0.00875F;
   IMU.gyro_Z=(float)Z*0.00875F;

   /*HMC5883L*/
   setdevicei2c(fd, HMC5883L_ADDRESS);
   buf=readbytei2c(fd, 0x03);
   X = buf<<8;
   buf=readbytei2c(fd, 0x04);
   X |= buf;
   if(X>32768){
      X=X-65535;
   }
   buf=readbytei2c(fd, 0x07);
   Y = buf<<8;
   buf=readbytei2c(fd, 0x08);
   Y |= buf;
   if(Y>32768){
      Y=Y-65535;
   }
   buf=readbytei2c(fd, 0x05);
   Z = buf<<8;
   buf=readbytei2c(fd, 0x06);
   Z |= buf;
   if(Z>32768){
      Z=Z-65535;
   }
   IMU.mag_X=(float)X/1090;
   IMU.mag_Y=(float)Y/1090;
   IMU.mag_Z=-(float)Z/1090;

   filterIMU(&IMU,reset);
   return IMU;
}
/*READ RANGE AND LIGHT VALUES (SONAR)*/
void sonar(int fd, int *range, int *light){
   unsigned char buf;
   setdevicei2c(fd,SRF08_ADDRESS);
   usleep(50000);
   writebytei2c(fd,0x00,0x51);
   usleep(100000);
   buf=readbytei2c(fd, 0x02);
   *range = buf<<8;
   buf=readbytei2c(fd, 0x03);
   *range |= buf;
   *light=readbytei2c(fd, 0x01);
}
/*SAVE RANFE AND LIGHT VALUE*/
void save_range_light(int light[],int range[]){
   FILE *fp;
   if((fp=fopen("/var/www/RGraph/data_range_light.csv","w"))==NULL){
      printf("Error to write measure.\n");
      //exit(1);
   }else{
      fprintf(fp,"%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d",light[0],light[1],light[2],light[3],light[4],light[5],light[6],light[7],light[8],light[9],light[10],light[11],range[0],range[1],range[2],range[3],range[4],range[5],range[6],range[7],range[8],range[9],range[10],range[11]);
      fclose(fp);
   }
}
/*CHASING LIGHT*/
int chasing_light_range(int mode,int fd_i2c,int *pos,int start,int stop,int light[],int range[]){
   int i;
   int min=0;
   int max=0;

   if(mode>4){
      printf("Error: Invalid mode.\n");
      return 0;
   }
   if((abs(start)+abs(stop)>14)||(start>stop)){
      printf("Error: Invalid range.\n");
      return 0;
   }
   for(i=0;i<=abs(start-stop);i++){
      motor_sonar_set(pos,i+start);
      sonar(fd_i2c,&range[i],&light[i]);
   }
   save_range_light(light,range);

   if(mode==0){
      motor_sonar_set(pos,0);
      return 0;
   }else if(mode==1){
      for(i=0;i<=abs(start-stop);i++){
         if(light[i]>light[max]){
            max=i;
         }
      }
      motor_sonar_set(pos,max+start);
      return max+start;
   }else if(mode==2){
      for(i=0;i<=abs(start-stop);i++){
         if(light[i]<light[min]){
            min=i;
         }
      }
      motor_sonar_set(pos,min+start);
      return min+start;
   }else if(mode==3){
      for(i=0;i<=abs(start-stop);i++){
         if(range[i]>range[max]){
            max=i;
         }
      }
      motor_sonar_set(pos,max+start);
      return max+start;
   }else if(mode==4){
      for(i=0;i<=abs(start-stop);i++){
         if(range[i]<range[min]){
            min=i;
         }
      }
      motor_sonar_set(pos,min+start);
      return min+start;
   }
}
/*CHASING RANGE*/
int chasing_range(int fd_i2c,int *pos,int start,int stop,int light[],int range[]){
   int i;
   int max=0;
   if((abs(start)+abs(stop)>14)||(start>stop)){
      printf("Error: Invalid range.\n");
      return 0;
   }
   for(i=0;i<=abs(start-stop);i++){
      motor_sonar_set(pos,i+start);
      sonar(fd_i2c,&range[i],&light[i]);
   }
   for(i=0;i<=abs(start-stop);i++){
      if(range[i]>range[max]){
         max=i;
      }
   }
   save_range_light(light,range);
   motor_sonar_set(pos,max+start);
   return max+start;
}
/*SEND AND RECEIVE SMS (DAISY13)*/
int sms(char mode, char *buffer){
   static long long int smsnumber=+0LL; //Default number
   char stringnum[20];
   char del[]=",.";
   char smskey[30];
   int op;
   char opstring[5];
   int num=-1;
   int num1;
   float credit;
   int i;
   FILE *fd;
   FILE *fd_sms;
   if((fd = fopen("/dev/ttyS2","w+"))==NULL){
      printf("Error: Daisy13 module (GPRS module) not found\n");
      //exit(1);
   }else{
      if(mode=='t'){
         rewind(fd);
         fprintf(fd,"AT\r\n");                           // Test AT command 
         usleep(200000);
         fprintf(fd,"AT+CMGF=1\r\n");                    // Chose text mode 
         usleep(200000);
         fprintf(fd,"AT+CMGD=1\r\n");                    // Delete the received message from memory position 1 
         usleep(200000);
         fprintf(fd,"AT+CMGS=\"%+Ld\"\r\n",smsnumber);   // Phone number to send 
         usleep(200000);
         fprintf(fd,"%s\n",buffer);                      // Text message 
         putc(0x1A,fd);                                  // Print (Ctr+Z) to send message 
         usleep(500000);
         rewind(fd);
         usleep(500000);
      }else if(mode=='r'){
         if((fd_sms = fopen("save/sms","r"))==NULL){
            printf("Error: SMS code list not found\n");
            //exit(1);
         }else{
            rewind(fd);
            for(i=0;i<strlen(buffer);i++){
               buffer[i]='\0';
            }
            fprintf(fd,"AT\r\n");                           // Test AT command
            usleep(200000);
            fprintf(fd,"AT+CMGF=1\r\n");                    // Chose text mode 
            usleep(200000);
            fprintf(fd,"AT+CNMI=1,2,0,0,0\r\n");            // Receive message
            usleep(200000);
            rewind(fd);
            usleep(200000);
            for(i=1;i<=4;i++){
               fscanf(fd,"%s",buffer);
               usleep(200000);
            }
            if(!strcmp(buffer,"OK")){
               printf("In attesa di ricevere un SMS... (premere ^C per terminare)\n");
            }
            do{
               fscanf(fd,"%s",buffer);
               usleep(500000);
            }while((!strcmp(buffer,"OK"))||(!strcmp(buffer,"AT"))||(buffer[0]=='>')||(buffer[2]=='+'));
            fscanf(fd,"%s",buffer);
            for(i=1;i<14;i++){
               stringnum[i-1]=buffer[i];
            }
            smsnumber=atoll(stringnum);
            fscanf(fd,"%s",buffer);
            strcat(buffer,",99999");
            strtok(buffer,del);
            num=atoi(strtok(NULL,del));
            while(!feof(fd_sms)){
               fscanf(fd_sms,"%d\t%d\t%s",&op,&num1,smskey);
               if(!strcmp(buffer,smskey)){
                  sprintf(opstring,"%02d",op);
                  strcpy(buffer,opstring);
                  if(num1!=99999){
                     num=num1;
                  }
                  fclose(fd_sms);
                  return num;
               }
            }
            fclose(fd_sms);
            strcpy(buffer,"XX");
            return -1;
         }
      }else if(mode=='c'){
         rewind(fd);
         fprintf(fd,"AT\r\n");                           // Test AT command
         usleep(200000);
         fprintf(fd,"AT+CMGF=1\r\n");                    // Chose text mode 
         usleep(200000);
         fprintf(fd,"AT+CNMI=1,2,0,0,0\r\n");            // Receive message
         usleep(200000);
         fprintf(fd,"AT+CMGD=1\r\n");                    // Delete the received message from memory position 1 
         usleep(200000);
         fprintf(fd,"AT+CMGS=\"40916\"\r\n");            // Phone number to send (40916)
         usleep(200000);
         fprintf(fd,"Credito\n");                        // Text message 
         putc(0x1A,fd);                                  // Print (Ctr+Z) to send message
         usleep(500000);
         printf("In attesa di ricevere la risposta da TIM... (premere ^C per terminare)\n");
         do{
            fscanf(fd,"%s",buffer);
            usleep(200000);
         }while((buffer[0]!='T')||(buffer[1]!='I')||(buffer[2]!='M'));
         for(i=1;i<=4;i++){
            fscanf(fd,"%s",buffer);
         }
         for(i=0;i<strlen(buffer);i++){
            if(buffer[i]==','){
               buffer[i]='.';
            }
         }
         credit=atof(buffer);
         return credit*100;
      }else{
         printf("Error: invalid mode. 't' = send SMS  'r' = receive SMS.\n");
         //exit(1);
      }
      fclose(fd);
      return num;
   }
}
/*OUTPUT ROUTINE*/
void printrobot(int m,int sd_client,char *buffer){
   if(m==1){
      send(sd_client,buffer,strlen(buffer),0);
   }else if(m==2){
      sms('t',buffer);
   }
   printf("%s",buffer);
}
/*MONITORING ENVIRONMENT*/
void monitoring_env(int fd_i2c, int time){
   FILE *fp;
   float temperature;
   float altitude;
   float pressure;
   int range;
   int light;
   for(;;){
      readTAP(fd_i2c,&temperature,&altitude,&pressure);
      sonar(fd_i2c,&range,&light);
      if((fp=fopen("/var/www/RGraph/data_env.csv","w"))==NULL){
         printf("Error to write measure.\n");
         //exit(1);
      }else{
         fprintf(fp,"0.0;%.2f;%.2f;%.2f;%d;%d",temperature,altitude,pressure,range,light);
         fclose(fp);
      }
      usleep(time*1000);
   }
}
/*MONITORING IMU*/
void monitoring_imu(int fd_i2c, int time){
   char buffer[50];
   struct structIMU IMU;
   struct sockaddr_in server_addr;
   struct sockaddr_in client_addr;
   int sd_server;
   int sd_client;

   if((sd_server = socket(AF_INET, SOCK_STREAM,0))<0){
      printf("Errore nella creazione del Server\n");
   }else{
      server_addr.sin_family = AF_INET;
      server_addr.sin_port = htons(4500);
      server_addr.sin_addr.s_addr = INADDR_ANY;
      int bindstatus=0;
      do{
         if(bind(sd_server,(struct sockaddr *)&server_addr, sizeof(server_addr))<0){
            bindstatus++;
            if(bindstatus==1){
               printf("Errore: Socket ancora occupato. Attendere alcuni secondi.\n\n");
            }else{
	           printf("\033[ARiconnessione... [Tentativo: %d]\n",bindstatus-1);
               sleep(3);
            }
         }else{
            bindstatus=0;
            listen(sd_server,1);
            socklen_t address_size = sizeof(client_addr);
            printf("In attesa di un client sulla porta %d... (premere ^C per terminare)\n",htobe16(server_addr.sin_port));
            if((sd_client = accept(sd_server, (struct sockaddr *)&client_addr, &address_size))<0){
               printf("Errore nella chiamata accept\n");
            }else{
               char *ip_address = inet_ntoa(client_addr.sin_addr);
               printf("Stabilita una connessione con il client %s\n",ip_address);
               printf("\033[35;1mMonitoraggio della posizione attivato\033[m\n");
            }
         }
      }while(bindstatus!=0);
   }
   for(;;){
      recv(sd_client, buffer,5,0);
      buffer[5]='\0';
      if(!strcmp(buffer,"RESET")){
         IMU=readIMU(fd_i2c,1);
         printf("\033[35;1mRotazioni resettate\033[m\n");
      }else if(!strcmp(buffer,"EXIT*")){
         close(sd_client);
	     close(sd_server);
         printf("\033[35;1mMonitoraggio della posizione interrotto\033[m\n");
         exit(0);
      }else{
         IMU=readIMU(fd_i2c,0);
      }
      sprintf(buffer,"%f,%f,%f\n",IMU.X,IMU.Y,IMU.Z);
      send(sd_client,buffer,strlen(buffer),0);
   }
}
/*REMOVE ECHO*/
unsigned char read_nocannon(int *c){
   /* pulisce il vecchio carattere presente in memoria */
   *c='\0';
   /* orig per ripristinare le impostazioni
    * new per cambiare le impostazioni */
   struct termios orig, new;
   /* Legge le impostazioni del terminale e
    * le memorizza in orig (vedere `man tcgetattr')*/
   if(tcgetattr(0, &orig) != 0){
      printf("tcgetattr");
      return 0;
   }
   /* copia le impostazioni in new. Metodo consigliato nella
    * documentazione della libreria GNU C (vedere `info libc') */
   memcpy(&new, &orig, sizeof(struct termios));
   /* Imposta `canonical imput' e toglie l'echo dei caratteri */
   new.c_lflag &= ~(ICANON|ECHO);
   /* Imposta il minimo numero di caratteri da leggere */
   new.c_cc[VMIN] = 1;
   /* Imposta il tempo di attesa 0 dice a read di attendere finche' non ci
    * sono X.c_cc[VMIN] caratteri letti */
   new.c_cc[VTIME] = 0;
   /* Setta le nuove impostazioni (vedere `man tcsetattr') */
   if(tcsetattr(0, TCSANOW, &new) != 0){
      printf("tcsetattr");
      return 0;
   }
   /* legge 1 carattere */
   if(read(0, c, 1) == -1){
      printf("read");
      tcsetattr(0, TCSANOW, &orig);
      return 0;
   }
   /* Ripristina le impostazioni precedenti ###IMPORTANTE### */
   tcsetattr(0, TCSANOW, &orig);
   return 1;
}
/*SOUND HORN*/
void horn(int time, int frequency){
   int i;
   for(i=0;i<(time/2);i++){
      gpio_wvalue(SOUND,1);
      usleep(frequency);
      gpio_wvalue(SOUND,0);
      usleep(frequency);
   }
}
/*SOUND HORN - GENERAL LEE DIXIE*/
void horn_dixie(){

//for General Lee horn
   int RE=1000-50;
   int DO=1250-50;
   int SI=1500-50;
   int LA=1750-50;
   int SOL=2000-50;
   int REP=100;
   int DOP=87;
   int SIP=75;
   int LAP=62;
   int SOLP=50;

   horn(REP,RE);
   usleep(50000);
   horn(SIP,SI);
   usleep(50000);
   horn(SOLP,SOL);
   usleep(50000);
   horn(SOLP,SOL);
   usleep(50000);
   horn(SOLP-10,SOL);
   usleep(50000);
   horn(LAP-10,LA);
   usleep(50000);
   horn(SIP-10,SI);
   usleep(50000);
   horn(DOP-10,DO);
   usleep(50000);
   horn(REP-10,RE);
   usleep(50000);
   horn(REP,RE);
   usleep(50000);
   horn(REP,RE);
   usleep(50000);
   horn(SIP,DO);
}
/*MOVE MOTOR AND SONAR*/
void motor_dc_forward(){
   gpio_wvalue(DCMOTOR2,0);
   gpio_wvalue(DCMOTOR1,1);
}
void motor_dc_back(){
   gpio_wvalue(DCMOTOR2,1);
   gpio_wvalue(DCMOTOR1,1);
}
void motor_dc_brake_forward(){
   gpio_wvalue(DCMOTOR2,1);
   gpio_wvalue(DCMOTOR1,1);
   usleep(TIMEDC/5);
   gpio_wvalue(DCMOTOR1,0);
}
void motor_dc_brake_back(){
   gpio_wvalue(DCMOTOR2,0);
   gpio_wvalue(DCMOTOR1,1);
   usleep(TIMEDC/5);
   gpio_wvalue(DCMOTOR1,0);
}
int motor_dc_set(int fd_i2c,int *pos,int step){
   int i=0;
   int memd=shmget(IPC_PRIVATE, sizeof(int), SHM_R | SHM_W | IPC_CREAT | IPC_EXCL);
   int *range=(int *)shmat(memd, 0, SHM_R | SHM_W);
   int light;

   if((step>0)&&(pos[0]<0)){
      motor_sonar_set(pos,-1);
   }else if((step<0)&&(pos[0]<0)){
      motor_sonar_set(pos,-5);
   }else if((step>0)&&(pos[0]<=-4)){
      motor_sonar_set(pos,-2);
   }else if((step<0)&&(pos[0]<=-4)){
      motor_sonar_set(pos,-4);
   }else if((step>0)&&(pos[0]==0)){
      motor_sonar_set(pos,0);
   }else if((step<0)&&(pos[0]==0)){
      motor_sonar_set(pos,6);
   }else if((step>0)&&(pos[0]<=3)){
      motor_sonar_set(pos,1);
   }else if((step<0)&&(pos[0]<=3)){
      motor_sonar_set(pos,5);
   }else if((step>0)&&(pos[0]<=7)){
      motor_sonar_set(pos,2);
   }else if((step<0)&&(pos[0]<=7)){
      motor_sonar_set(pos,4);
   }

   int rangechildpid=fork();
   if(rangechildpid==0){
      for(;;){
         sonar(fd_i2c,range,&light);
         usleep(100000);
      }
   }
   usleep(200000);
   if((step<=50)&&(step>=-50)){
      if(step>0){
         motor_dc_forward();
         while((i<step)&&(*range>17+(i*i/2))){
            if(gpio_rvalue(REVCOUNTER)){
               i++;
               printf("Step: %d   Distanza Sonar: %d\n\033[1A",i,*range);
               while(gpio_rvalue(REVCOUNTER));
            }
            usleep(25000);
         }
         motor_dc_brake_forward();
      }else if(step<0){
         motor_dc_back();
         while((i>step)&&(*range>17+(i*i/2))){
            if(gpio_rvalue(REVCOUNTER)){
               i--;
               printf("Step: %d   Distanza Sonar: %d\n\033[1A",i,*range);
               while(gpio_rvalue(REVCOUNTER));
            }
            usleep(25000);
         }
         motor_dc_brake_back();
      }
      printf("\n");
      kill(rangechildpid,SIGTERM);
      return i;
   }else{
      kill(rangechildpid,SIGTERM);
      return -90;
   }
}

void motor_stepper_right(){
   gpio_wvalue(SMOTORA1,0);
   gpio_wvalue(SMOTORA2,1);
   gpio_wvalue(SMOTORB1,0);
   gpio_wvalue(SMOTORB2,0);
   usleep(TIME);
   gpio_wvalue(SMOTORA1,0);
   gpio_wvalue(SMOTORA2,0);
   gpio_wvalue(SMOTORB1,1);
   gpio_wvalue(SMOTORB2,0);
   usleep(TIME);
   gpio_wvalue(SMOTORA1,1);
   gpio_wvalue(SMOTORA2,0);
   gpio_wvalue(SMOTORB1,0);
   gpio_wvalue(SMOTORB2,0);
   usleep(TIME);
   gpio_wvalue(SMOTORA1,0);
   gpio_wvalue(SMOTORA2,0);
   gpio_wvalue(SMOTORB1,0);
   gpio_wvalue(SMOTORB2,1);
   usleep(TIME);
   gpio_wvalue(SMOTORA1,0);
   gpio_wvalue(SMOTORA2,0);
   gpio_wvalue(SMOTORB1,0);
   gpio_wvalue(SMOTORB2,0);
}
void motor_stepper_left(){
   gpio_wvalue(SMOTORA1,0);
   gpio_wvalue(SMOTORA2,1);
   gpio_wvalue(SMOTORB1,0);
   gpio_wvalue(SMOTORB2,0);
   usleep(TIME);
   gpio_wvalue(SMOTORA1,0);
   gpio_wvalue(SMOTORA2,0);
   gpio_wvalue(SMOTORB1,0);
   gpio_wvalue(SMOTORB2,1);
   usleep(TIME);
   gpio_wvalue(SMOTORA1,1);
   gpio_wvalue(SMOTORA2,0);
   gpio_wvalue(SMOTORB1,0);
   gpio_wvalue(SMOTORB2,0);
   usleep(TIME);
   gpio_wvalue(SMOTORA1,0);
   gpio_wvalue(SMOTORA2,0);
   gpio_wvalue(SMOTORB1,1);
   gpio_wvalue(SMOTORB2,0);
   usleep(TIME);
   gpio_wvalue(SMOTORA1,0);
   gpio_wvalue(SMOTORA2,0);
   gpio_wvalue(SMOTORB1,0);
   gpio_wvalue(SMOTORB2,0);
}
int motor_stepper_set(int *pos, int step){
   if((step>=-7)&&(step<=7)){
      if(pos[0]<step){
         for(;pos[0]<step;pos[0]++){
            motor_stepper_right();
            usleep(TIME);
         }
      }else if(pos[0]>step){
         for(;pos[0]>step;pos[0]--){
            motor_stepper_left();
            usleep(TIME);
         }
      }
      FILE *fd_motor;
      if((fd_motor=fopen("save/motor","w"))!=NULL){
         fprintf(fd_motor,"%d,%d",pos[0],pos[1]);
         fclose(fd_motor);
      }else{
         printf("Errore: Impossibile salvare le posizioni dei motori\n");
      }
      return 0;
   }else{
      return -1;
   }
}
void motor_sonar_right(){
   gpio_wvalue(SONARA1,0);
   gpio_wvalue(SONARA2,0);
   gpio_wvalue(SONARB1,1);
   gpio_wvalue(SONARB2,0);
   usleep(TIME);
   gpio_wvalue(SONARA1,0);
   gpio_wvalue(SONARA2,1);
   gpio_wvalue(SONARB1,0);
   gpio_wvalue(SONARB2,0);
   usleep(TIME);
   gpio_wvalue(SONARA1,0);
   gpio_wvalue(SONARA2,0);
   gpio_wvalue(SONARB1,0);
   gpio_wvalue(SONARB2,1);
   usleep(TIME);
   gpio_wvalue(SONARA1,1);
   gpio_wvalue(SONARA2,0);
   gpio_wvalue(SONARB1,0);
   gpio_wvalue(SONARB2,0);
   usleep(TIME);
   gpio_wvalue(SONARA1,0);
   gpio_wvalue(SONARA2,0);
   gpio_wvalue(SONARB1,0);
   gpio_wvalue(SONARB2,0);
}
void motor_sonar_left(){
   gpio_wvalue(SONARA1,0);
   gpio_wvalue(SONARA2,0);
   gpio_wvalue(SONARB1,0);
   gpio_wvalue(SONARB2,1);
   usleep(TIME);
   gpio_wvalue(SONARA1,0);
   gpio_wvalue(SONARA2,1);
   gpio_wvalue(SONARB1,0);
   gpio_wvalue(SONARB2,0);
   usleep(TIME);
   gpio_wvalue(SONARA1,0);
   gpio_wvalue(SONARA2,0);
   gpio_wvalue(SONARB1,1);
   gpio_wvalue(SONARB2,0);
   usleep(TIME);
   gpio_wvalue(SONARA1,1);
   gpio_wvalue(SONARA2,0);
   gpio_wvalue(SONARB1,0);
   gpio_wvalue(SONARB2,0);
   usleep(TIME);
   gpio_wvalue(SONARA1,0);
   gpio_wvalue(SONARA2,0);
   gpio_wvalue(SONARB1,0);
   gpio_wvalue(SONARB2,0);
}
int motor_sonar_set(int *pos, int step){
   if((step>=-7)&&(step<=7)){
      if(pos[1]<step){
         for(;pos[1]<step;pos[1]++){
            motor_sonar_right();
            usleep(TIME);
         }
      }else if(pos[1]>step){
         for(;pos[1]>step;pos[1]--){
            motor_sonar_left();
            usleep(TIME);
         }
      }
      FILE *fd_motor;
      if((fd_motor=fopen("save/motor","w"))!=NULL){
         fprintf(fd_motor,"%d,%d",pos[0],pos[1]);
         fclose(fd_motor);
      }else{
         printf("Errore: Impossibile salvare le posizioni dei motori\n");
      }
      return 0;
   }else{
      return -1;
   }
}
void motor_stop(){
   gpio_wvalue(SMOTORA1,0);
   gpio_wvalue(SMOTORA2,0);
   gpio_wvalue(SMOTORB1,0);
   gpio_wvalue(SMOTORB2,0);
   gpio_wvalue(SONARA1,0);
   gpio_wvalue(SONARA2,0);
   gpio_wvalue(SONARB1,0);
   gpio_wvalue(SONARB2,0);
}
void movement(int mode, int sd_client, int *pos){
   int RE=1000-50;
   int mov;
   int dir=0;
   //0: stopped
   //1: forward
   //2: back
   int proximitychildpid=0;
   //0: figlio da creare
   //!=0: figlio gia' creato

   int hornchildpid=0;
   //0: figlio da creare
   //!=0: figlio gia' creato

   gpio_open(DCMOTOR1,'o');
   gpio_open(DCMOTOR2,'o');
   gpio_open(SMOTORA1,'o');
   gpio_open(SMOTORA2,'o');
   gpio_open(SMOTORB1,'o');
   gpio_open(SMOTORB2,'o');
   gpio_open(SONARA1,'o');
   gpio_open(SONARA2,'o');
   gpio_open(SONARB1,'o');
   gpio_open(SONARB2,'o');
   gpio_open(REVCOUNTER,'i');
   gpio_open(PROXSE,'i');
   gpio_open(PROXSW,'i');
   gpio_open(PROXNE,'i');
   gpio_open(PROXNW,'i');
   gpio_open(SOUND,'o');
   gpio_wvalue(DCMOTOR1,0);
   gpio_wvalue(DCMOTOR2,0);
   motor_stop();
   for(;;){
      if(mode==1){
         int i;
         char buffer[150];
         sprintf(buffer,"Posizione motore stepper: %d   Posizione sonar: %d\n",pos[0],pos[1]);
         send(sd_client,buffer,strlen(buffer),0);
         for(i=0;i<150;i++){
            buffer[i]='\0';
         }
         recv(sd_client, buffer, sizeof(buffer),0);
         printf("\033[33;1mRicevuto il comando '%c'\033[m\n",buffer[0]);
         mov=buffer[0]; 
      }else{
         read_nocannon(&mov);
      }
      if((mov==32)||(mov==' ')){
         if(proximitychildpid!=0){
            kill(proximitychildpid,SIGTERM);
            gpio_wvalue(DCMOTOR1,0);
            gpio_wvalue(DCMOTOR2,0);
            proximitychildpid=0;
         }
         if(dir==1){   
            motor_dc_brake_forward();
         }else if(dir==2){
            motor_dc_brake_back();
         }
         dir=0;
      }
      if((mov==65)||(mov=='w')){
         motor_dc_forward();
         dir=1;
      }
      if((mov==66)||(mov=='s')){
         motor_dc_back();
         dir=2;
      }
      if(mov=='c'){
         if(hornchildpid!=0){
            kill(hornchildpid,SIGTERM);
            gpio_wvalue(SOUND,0);
            hornchildpid=0;
         }else{
            hornchildpid=fork();
            if(hornchildpid==0){
               horn(1000000,RE);
            }
         }
      }
      if(mov=='p'){
         if(proximitychildpid==0){
            proximitychildpid=fork();
            if(proximitychildpid==0){
               while(1){
                  if((dir==1)||(dir==0)){
                     motor_dc_forward();
                     while((gpio_rvalue(PROXSE))&&(gpio_rvalue(PROXSW))){
                     }
                     if((gpio_rvalue(PROXNE))&&(gpio_rvalue(PROXNW))){
                        gpio_wvalue(DCMOTOR2,1);
                        usleep(100*1000);
                     }
                     gpio_wvalue(DCMOTOR1,0);
                     dir=2;
                  }else{
                     motor_dc_back();
                     while((gpio_rvalue(PROXNE))&&(gpio_rvalue(PROXNW))){
                     }
                     if((gpio_rvalue(PROXNE))&&(gpio_rvalue(PROXNW))){
                        gpio_wvalue(DCMOTOR2,0);
                        usleep(100*1000);
                     }
                     gpio_wvalue(DCMOTOR1,0);
                     dir=1;
                  }
               }
            }
         }else{
            kill(proximitychildpid,SIGTERM);
            gpio_wvalue(DCMOTOR1,0);
            gpio_wvalue(DCMOTOR2,0);
            proximitychildpid=0;
         }
      }
      if(mov=='r'){
         pos[0]=0;
         pos[1]=0;
      }
      if(((mov==67)||(mov=='d'))&&(pos[0]<7)){
         pos[0]++;
         motor_stepper_right();
      }    
      if(((mov==68)||(mov=='a'))&&(pos[0]>-7)){
         pos[0]--;
         motor_stepper_left();
      }
      if((mov=='l')&&(pos[1]<7)){
         pos[1]++;
         motor_sonar_right();
      }
      if((mov=='k')&&(pos[1]>-7)){
         pos[1]--;
         motor_sonar_left();
      }
      if(mov=='x'){
         motor_stop();
         FILE *fd_motor;
         if((fd_motor=fopen("save/motor","w"))!=NULL){
            fprintf(fd_motor,"%d,%d",pos[0],pos[1]);
            fclose(fd_motor);
         }else{
            printf("Errore: Impossibile salvare le posizioni dei motori\n");
         }
         if(hornchildpid!=0){
            kill(hornchildpid,SIGTERM);
            gpio_wvalue(SOUND,0);
            hornchildpid=0;
         }
         if(proximitychildpid!=0){
            kill(proximitychildpid,SIGTERM);
            gpio_wvalue(DCMOTOR1,0);
            gpio_wvalue(DCMOTOR2,0);
            proximitychildpid=0;
         }
         printf("\n");
         exit(0);
      }
      motor_stop();
   }
}


int gmtoffset(void) {
  struct tm *dummy;
  time_t t = 0;
  dummy = localtime(&t);
  return dummy->tm_gmtoff; /* _BSD_SOURCE */
}


/*GPS DATA*/
struct structGPS gps(){
   struct structGPS GPS;
   char buffer[150];
   int read=0;
   char temp[10];
   FILE *fd;
   if((fd = fopen("/dev/ttyS3","w+"))==NULL){
      printf("Error: Daisy7 module (MEMS-GPS module) not found\n");
      //exit(1);
   }else{
      while(read==0){
         fscanf(fd,"%s",buffer);
         if((buffer[0]=='$')&&(buffer[1]=='G')&&(buffer[4]=='G')){
            time_t rawtime;
            struct tm *info;
            char currenttime[35];
            strcpy(GPS.nmea,buffer);
            sscanf(buffer,"%6s,%c%c%c%c%c%c.%3s,%f,%c,%f,%c,%d,%d%5c%f%s",temp,&GPS.time_hms[0],&GPS.time_hms[1],&GPS.time_hms[3],&GPS.time_hms[4],&GPS.time_hms[6],&GPS.time_hms[7],temp,&GPS.latitude,&GPS.latitude_d,&GPS.longitude,&GPS.longitude_d,&GPS.quality,&GPS.satellite,temp,&GPS.altitude,temp);
            GPS.time_hms[2]=':';
            GPS.time_hms[5]=':';
            GPS.time_hms[8]='\0';
            time(&rawtime);
            info = localtime( &rawtime );
            strftime(currenttime, sizeof(currenttime), "%T", info);
            putenv("TZ=UTC");
            strftime(GPS.time_hms, sizeof(GPS.time_hms), "%T", info);
            sprintf(buffer,"date -s %s > /dev/null",GPS.time_hms);
            system(buffer);
            putenv("TZ=Europe/Rome");
            sprintf(buffer,"date -s %s > /dev/null",currenttime);
            system(buffer);
            read=1;
         }
         usleep(200000);
      }
   }
   return GPS;
}
/*GPS COORDINATE FOR GOOGLE MAPS*/
struct structGPS gps_google(){
   struct structGPS GPS;
   int lattemp;
   int lontemp;
   float lattemp1;
   float lontemp1;
   GPS=gps();
   lattemp=(int)GPS.latitude/100;
   lontemp=(int)GPS.longitude/100;
   lattemp1=GPS.latitude-lattemp*100;
   lontemp1=GPS.longitude-lontemp*100;
   GPS.latitude_g=lattemp+lattemp1/60;
   GPS.longitude_g=lontemp+lontemp1/60;
   return GPS;
}
/*ITALIAN DATE AND TIME*/
char* data_ora(char *buffer){
   char *mesi[] = {"Gennaio", "Febbraio", "Marzo", "Aprile", "Maggio", "Giugno","Luglio", "Agosto", "Settembre", "Ottobre", "Novembre", "Dicembre"};
   char *giorni[] = {"Domenica", "Lunedi'", "Martedi'", "Mercoledi'", "Giovedi'", "Venerdi'", "Sabato"};
   time_t rawtime;
   time (&rawtime);
   struct tm* leggibile;
   leggibile = localtime(&rawtime);
   int i;
   for(i=0;i<70;i++){
      buffer[i]='a';
   } 
   snprintf(buffer,50,"%s %d %s %d  %02d:%02d:%02d                      ",giorni[leggibile->tm_wday],leggibile->tm_mday,mesi[leggibile->tm_mon],leggibile->tm_year + 1900,leggibile->tm_hour,leggibile->tm_min,leggibile->tm_sec);
   return buffer;
}
/*SAVE DATA ON DATABASE*/
void database(char mode,int fd_i2c){
   MYSQL *conn;
   char query[500];
   conn=mysql_init(NULL);
   mysql_real_connect (conn,"localhost","root","netusg20","robot",0,NULL,0);
   if(mode=='e'){
      float temperature;
      float altitude;
      float pressure;
      int range;
      int light;
      readTAP(fd_i2c,&temperature,&altitude,&pressure);
      sonar(fd_i2c,&range,&light);
      sprintf(query,"INSERT INTO robot.Misurazioni_Ambientali VALUES (CURRENT_TIMESTAMP,%f,%f,%f,%d,%d);",temperature,altitude,pressure,range,light);
   }else if(mode=='i'){
      struct structIMU IMU=readIMU(fd_i2c,0);
      sprintf(query,"INSERT INTO robot.Misurazioni_Inerziali VALUES (CURRENT_TIMESTAMP,%f,%f,%f,%f,%f,%f,%f,%f,%f);",IMU.acc_X,IMU.acc_Y,IMU.acc_Z,IMU.gyro_X,IMU.gyro_Y,IMU.gyro_Z,IMU.mag_X,IMU.mag_Y,IMU.mag_Z);
   }else if(mode=='g'){
      struct structGPS GPS=gps_google();
      sprintf(query,"INSERT INTO robot.GPS VALUES (CURRENT_TIMESTAMP,'%s',%d,'%s','%f%c','%f%c',%f,%f,%f,%d);",GPS.nmea,GPS.quality,GPS.time_hms,GPS.latitude,GPS.latitude_d,GPS.longitude,GPS.longitude_d,GPS.latitude_g,GPS.longitude_g,GPS.altitude,GPS.satellite);
   }else{
      printf("Error: Invalid choice\n");
   }
   mysql_query(conn, query);
   mysql_close (conn);
}



/////////////////
//LAVORI IN CORSO"
/////////////////
//LAVORI IN CORSO
/////////////////
//LAVORI IN CORSO
/////////////////
//LAVORI IN CORSO
/////////////////
//LAVORI IN CORSO

/*FOLLOWING LIGHT*/  /*LAVORI IN CORSO*/
int following_light(int fd_i2c,int *pos){
   int lightv[15];
   int rangev[15];
   int lightpos;
   int stepperpos;
   int range;
   int light[3];
//???????????
   lightpos=chasing_light_range(0,fd_i2c,pos,-6,5,lightv,rangev);
   for(;;){
      if((lightpos==0)||(lightpos==6)){       
         lightpos=chasing_light_range(0,fd_i2c,pos,-6,5,lightv,rangev);
         if((lightpos==0)||(lightpos==6)){
            motor_stepper_set(pos,0);
            return lightpos;
         }
      }else if((lightpos>0)&&(lightpos<=3)){
         motor_stepper_set(pos,3);
         motor_dc_set(fd_i2c,pos,5);
      }else if((lightpos>3)&&(lightpos<=5)){
         motor_stepper_set(pos,-3);
         motor_dc_set(fd_i2c,pos,-5);
      }else if((lightpos<0)&&(lightpos>=-3)){
         motor_stepper_set(pos,3);
         motor_dc_set(fd_i2c,pos,5);
      }else if((lightpos<3)&&(lightpos>=-5)){
         motor_stepper_set(pos,-3);
         motor_dc_set(fd_i2c,pos,-5);
      }
      lightpos=chasing_light_range(0,fd_i2c,pos,lightpos-1,lightpos+1,lightv,rangev);
      usleep(500000);
     
//
/**/
   }
   return lightpos;
}

void movement_casual(int fd_i2c,int *pos){
   int i;
   int finalstep=10;
   int r;
   for(i=0;i<15;i++){
      if(finalstep==10){      
         finalstep=motor_dc_set(fd_i2c,pos,10);
      }else if(finalstep==-10){
         finalstep=motor_dc_set(fd_i2c,pos,-10);
      }else if(finalstep>=0){
         r=(rand()%3-1)*2;
         motor_stepper_set(pos,r);
         finalstep=-10;
      }else if(finalstep<0){
         r=(rand()%3-1)*2;
         motor_stepper_set(pos,r);
         finalstep=10;
      }
      usleep(500000);
   }
   motor_stepper_set(pos,0);
   motor_sonar_set(pos,0);
}
